#include <stdio.h>
#include <unistd.h>
#include <usb.h>

#define USB_RT_PORT  (USB_TYPE_CLASS | USB_RECIP_OTHER)
#define USB_PORT_FEAT_POWER  8
#define Sleep(msec)  usleep((msec)*1000)

int hub_paw_control_test(int vid,int pid)
{
	int i,found;
	usb_dev_handle *hnd = NULL;
	struct usb_bus *bus;
	struct usb_device *dev;
	int request, value, index,timeout;
	char buf[16];
	
	printf("Toro-Hub Paw Contorol Test\n");
	
	found=0;
	for (bus = usb_get_busses(); bus; bus = bus->next) {
		for (dev = bus->devices; dev; dev = dev->next) {
			if ((dev->descriptor.idVendor==vid)&&(dev->descriptor.idProduct==pid)) {
				found=1; break; 
			}
		}
		if (found==1) { break; }
	}
	if (found==0) {
		printf("Error: device not found.\n"); return 1;
	}
	if (!(hnd = usb_open(dev))) {
		printf("Error: device cannot open.\n"); return 1;
	}

	if (usb_set_configuration(hnd, 1) < 0) {
		printf("Error: setting config 1 failed.\n"); 
		printf(" (Do you have root permission?)\n"); 
		usb_close(hnd);
		return 1;
	}
	if (usb_claim_interface(hnd, 0) < 0) {
		printf("Error: claiming interface 0 failed.\n");
		usb_close(hnd);
		return 1;
	}
	
	timeout=5000;
	index = 0;
	request = 0; // Toro's motion number
	value = 100; // motion time
	for (i=0;i<5;i++) {
		request = 1;
		usb_control_msg(hnd, USB_TYPE_VENDOR, request, value, index, buf, 0, timeout);
		Sleep(500);
		request = 2;
		usb_control_msg(hnd, USB_TYPE_VENDOR, request, value, index, buf, 0, timeout);
		Sleep(500);
	}
	for (i=0;i<5;i++) {
		request = 3;
		usb_control_msg(hnd, USB_TYPE_VENDOR, request, value, index, buf, 0, timeout);
		Sleep(200);
	}
	
	usb_release_interface(hnd, 0);
	usb_close(hnd);
}

int hub_pow_control_test(int vid,int pid)
{
	int i,found;
	usb_dev_handle *hnd = NULL;
	struct usb_bus *bus;
	struct usb_device *dev;
	int request, value, index,timeout;
	char buf[16];
	
	printf("Toro-Hub Power Contorol Test\n");
	
	found=0;
	for (bus = usb_get_busses(); bus; bus = bus->next) {
		for (dev = bus->devices; dev; dev = dev->next) {
			if ((dev->descriptor.idVendor==vid)&&(dev->descriptor.idProduct==pid)) {
				found=1; break; 
			}
		}
		if (found==1) { break; }
	}
	if (found==0) {
		printf("Error: device not found.\n"); return 1;
	}
	if (!(hnd = usb_open(dev))) {
		printf("Error: device cannot open.\n"); return 1;
	}
	
	timeout=5000;
	value = USB_PORT_FEAT_POWER;
	for (i=0;i<5;i++) {
		index = 1; // hub port number
		request = USB_REQ_SET_FEATURE; // Power On
		usb_control_msg (hnd, USB_RT_PORT, request, value, index, buf, 0, timeout);
		index = 2; // hub port number
		request = USB_REQ_CLEAR_FEATURE; // Power Off
		usb_control_msg (hnd, USB_RT_PORT, request, value, index, buf, 0, timeout);
		Sleep(1000);
		index = 1; // hub port number
		request = USB_REQ_CLEAR_FEATURE; // Power Off
		usb_control_msg (hnd, USB_RT_PORT, request, value, index, buf, 0, timeout);
		index = 2; // hub port number
		request = USB_REQ_SET_FEATURE; // Power On
		usb_control_msg (hnd, USB_RT_PORT, request, value, index, buf, 0, timeout);
		Sleep(1000);
	}
	request = USB_REQ_CLEAR_FEATURE; // Power Off
	index = 1; // hub port number
	usb_control_msg (hnd, USB_RT_PORT, request, value, index, buf, 0, timeout);
	index = 2; // hub port number
	usb_control_msg (hnd, USB_RT_PORT, request, value, index, buf, 0, timeout);
	index = 3; // hub port number
	usb_control_msg (hnd, USB_RT_PORT, request, value, index, buf, 0, timeout);
	// Cannot contorol Toro-hub port3 power?

	usb_release_interface(hnd, 0);
	usb_close(hnd);
}

int main(void)
{
	int b,d;
	int VendorID,ProductID;
	
	usb_init(); // Initialize
	b = usb_find_busses(); // find all busses
	//printf("USB Bus count = %d\n",b);
	d = usb_find_devices(); // find all connected devices
	//printf("USB Device count = %d\n",d);
	
	VendorID = 0x054D; ProductID = 0x1B59;
	hub_paw_control_test(VendorID,ProductID);
	
	VendorID = 0x054D; ProductID = 0x1000;
	hub_pow_control_test(VendorID,ProductID);
	
	VendorID = 0x054D; ProductID = 0x1B59;
	hub_paw_control_test(VendorID,ProductID);
	
	return 0;
}
